from config import Configuration
from file_finder import FileFinder
from converto import Converto

import logging
import argparse


def main():
    args = parse_args()
    setup_logging(args.debug)
    configuration = Configuration(args.config)
    logging.debug(configuration)
    c = Converto(configuration, args.input)

def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("-d", "--debug", action='store_true', help="Show debugging information")
    parser.add_argument("-c", "--config", default=None, help="Specify Configuration File Path")
    parser.add_argument("-i", "--input", default=None, help="specify the file, or the directory of files, to operate on")
    args = parser.parse_args()
    return args

def setup_logging(debug):
    if debug:
        lvl = logging.DEBUG
    else:
        lvl = logging.INFO
    logging.basicConfig(level=logging.DEBUG)


if __name__ == "__main__":
    main()



from file_finder import FileFinder
from menu import Menu
from ffmpy import FFmpeg
from os import path, remove


class Converto:
    menu_options = list()
    files = list()
    config = None
    main_menu = None
    satisfied_menu = None
    chosen_option = None
    user_input = None

    def __init__(self, config, user_input):
        self.config = config
        self.input = user_input
        self.show_main_menu()

    def _convert(self):
        for f in self.files:
            for i, command in enumerate(self.chosen_option.commands):
                input_file = f
                is_intermediary = self._is_intermediary(i)
                previous_intermediary_exists = False

                previous_intermediary_file = self._get_output_filename(
                    f, (i - 1))
                if path.isfile(previous_intermediary_file):
                    previous_intermediary_exists = True

                if is_intermediary and previous_intermediary_exists:
                    input_file = previous_intermediary_file

                output_filename = self._get_output_filename(f, i)

                if self.chosen_option.multi_input:
                    inputs = {}
                    for f in self.files:
                            inputs[f] = command.input_options
                else:
                    inputs = {input_file: command.input_options}
                outputs = {output_filename: command.output_options}

                ff = FFmpeg(
                    inputs=inputs,
                    outputs=outputs
                )
                ff.run()

                if previous_intermediary_exists:
                    remove(previous_intermediary_file)
                elif self.chosen_option.multi_input:
                    return

    def _ask_if_commands_look_right(self):
        command_list = ""
        for command in self.chosen_option.commands:
            ff = FFmpeg(
                inputs={"{input_filename}": command.input_options},
                outputs={"{output_filename}": command.output_options}
            )
            command_list = command_list + ff.cmd + "\n"
        menu_title = "About to process these files: {0}\nUsing these commands:\n{1}\n\nDoes this look right?".format(
            self.files, command_list)
        self.satisfied_menu = Menu(title=menu_title)
        self.satisfied_menu.set_options([
            ("Yes, process the files now",
             lambda: self._handle_user_satisfaction_choice(True)),
            ("No, cancel", lambda: self._handle_user_satisfaction_choice(False)),
        ])
        self.satisfied_menu.open()

    def _handle_user_satisfaction_choice(self, satisified):
        self.satisfied_menu.close()
        if satisified:
            self._convert()
        else:
            exit(0)

    def show_main_menu(self):
        menu_options = self._generate_menu_options()
        self.main_menu = Menu(
            title="Choose which command you would like to run")
        self.main_menu.set_options(menu_options)
        self.main_menu.open()

    def _generate_menu_options(self):
        menu_options = list()
        for i, opt in enumerate(self.config.options):
            menu_options.append(
                (opt.name, lambda i=i: self._find_files_and_convert(i)))
        return menu_options

    def _find_files_and_convert(self, option_index):
        self.main_menu.close()
        self.chosen_option = self.config.options[option_index]
        print "You chose \"{0}\"\n".format(self.chosen_option.name)
        file_finder = FileFinder(self.chosen_option, self.user_input)
        self.files = file_finder.files
        print self.files
        self._ask_if_commands_look_right()

    def _get_output_filename(self, filename, command_index):
        command = self.chosen_option.commands[command_index]
        if self._is_intermediary(command_index):
            output_filename = "{0}_INTERMEDIARY_{1}.{2}".format(
                filename[:-4], command_index, command.output_extension)
        elif command.output_filename_format:
            output_filename = command.output_filename_format.format(
                input_filename=filename, extension=command.output_extension)
        else:
            output_filename = "{0}.{1}".format(
                filename[:-4], command.output_extension)
        return output_filename

    def _is_intermediary(self, index):
        return index != len(self.chosen_option.commands) - 1



import json
from os import path, getcwd


class Configuration:
    config_file_path = None
    options = None

    def __init__(self, config_file_path=None):
        self._find_config(config_file_path)
        self._parse_options()

    def __repr__(self):
        return "Config File: {0}\nOptions: {1}".format(self.config_file_path, self.options)

    def _find_config(self, config_file_path):
        if config_file_path:
            self.config_file_path = config_file_path
        else:
            self.config_file_path = path.join(
                __file__, "..", "configuration/configuration.json")
        if not path.exists(self.config_file_path):
            raise Exception("Failed to find configuration file at: {0}.".format(
                self.config_file_path))

    def _parse_options(self):
        options = list()
        with open(self.config_file_path, 'r') as config_file:
            config = json.load(config_file)
        for opt in config["options"]:
            commands = list()
            for com in opt["commands"]:
                commands.append(
                    Command(
                        self._get_attribute(com, "input-options"),
                        self._get_attribute(com, "output-options"),
                        self._get_attribute(com, "output-extension"),
                        self._get_attribute(com, "output-filename-format")
                    ))
            option = Option(
                self._get_attribute(opt, "name"),
                self._get_attribute(opt, "valid-input-extensions"),
                self._get_attribute(opt, "multi-input"),
                commands
            )
            options.append(option)
        self.options = options

    def _get_attribute(self, elem, attribute_name):
        try:
            return elem[attribute_name]
        except:
            return None


class Option:
    name = None
    valid_input_exts = None
    commands = None

    def __init__(self, name, valid_input_exts, multi_input, commands):
        self.name = name
        self.valid_input_exts = valid_input_exts
        self.multi_input = multi_input
        self.commands = commands

    def __repr__(self):
        return "Name: {0} | Valid Input Exts: {1}\nMulti Input: {2}\nCommands: {3}".format(self.name, self.valid_input_exts, self.multi_input, self.commands)


class Command:
    input_options = None
    output_options = None
    output_extension = None
    output_filename_format = None

    def __init__(self, input_options, output_options, output_extension, output_filename_format):
        self.input_options = input_options
        self.output_options = output_options
        self.output_extension = output_extension
        self.output_filename_format = output_filename_format

    def __repr__(self):
        return "Input Opts: {0} | Output Opts: {1} | Output Ext: {2} | Output Filename Format: {3}\n".format(self.input_options, self.output_options, self.output_extension, self.output_filename_format)



from os import path, walk
from menu import Menu

class FileFinder:
    files = list()
    extension_to_find = None
    option = None
    user_satisfied = False
    satisfied_menu = None

    def __init__(self, option, user_input=None):
        self.option = option
        self._get_user_input(user_input)

    def _get_user_input(self, _user_input):
        while not self.user_satisfied:
            if _user_input:
                user_input = _user_input
                _user_input = None
            else:
                user_input = self._ask_user_for_files()
            if path.isfile(user_input):
                if self._file_is_right_extension(user_input):
                    self.files.append(user_input)
            elif path.isdir(user_input):
                self._find_files_from_directory(user_input)
            else:
                print "The path entered: {0} is not valid.\nRetrying...".format(user_input)
                continue
            if len(self.files) == 0:
                print "Selection must contain at least one file of extension types: {0}".format(self._get_exts_list())
                continue
            self._ask_if_user_is_satisfied_with_files()

    def _ask_user_for_files(self):
        print "Valid input types: {0}\n".format(self._get_exts_list())
        return raw_input("Enter the path of your file or directory of files: ").strip()

    def _find_files_from_directory(self, user_input):
        for root, dirs, files in walk(user_input):
            for f in files:
                if self._file_is_right_extension(f):
                    self.files.append(path.join(root,f))

    def _ask_if_user_is_satisfied_with_files(self):
        satisfied = True
        self.satisfied_menu = Menu(title = "Files to Process:\n{0}\nDoes this look right?".format(self._get_file_list()))
        self.satisfied_menu.set_options([
            ("Yes, continue", lambda: self._handle_user_satisfaction_choice(True)),
            ("No, let me try again.", lambda: self._handle_user_satisfaction_choice(False)),
            ("Add another file.", lambda: self._handle_user_satisfaction_choice(True, add_another=True)),
        ])
        self.satisfied_menu.open()

    def _handle_user_satisfaction_choice(self, satisfied, add_another=False):
        self.satisfied_menu.close()
        if add_another:
            return
        if not satisfied:
            self.files = list()
        self.user_satisfied = satisfied
        
    def _get_file_list(self):
        file_list_string = ''
        for x in self.files:
            file_list_string = file_list_string + x + "\n"
        return file_list_string 

    def _get_exts_list(self):
        extension_list_string = ''
        for x in self.option.valid_input_exts:
            extension_list_string = extension_list_string + x + ", "
        return extension_list_string[:-2]

    def _file_is_right_extension(self, file_path):
        return file_path.endswith(tuple(self.option.valid_input_exts))


from distutils.core import setup

setup(
    name='Converto',
    version='0.1.5',
    packages=['converto'],
    license='MIT',
    entry_points={
        'console_scripts': ['converto=converto.command_line:main'],
    },
    author='Erik Hauck',
    description="A simple, configurable wrapper for FFmpeg",
    url='https://github.com/erikmhauck/converto',
    install_requires=[
        'menu',
        'ffmpy'
    ],
    include_package_data=True,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: End Users/Desktop',
        'Topic :: Multimedia',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7'
    ],
    keywords='ffmpeg wrapper configurable tool',
    project_urls={
        'Documentation': 'https://github.com/erikmhauck/converto',
        'Source': 'https://github.com/erikmhauck/converto',
        'Tracker': 'https://github.com/erikmhauck/converto/issues',
    },
    python_requires='>=2.7, <3',
    package_data={
        'converto': ['configuration/configuration.json'],
    },
)
